/*!
 * @file shell_command.h
 * Definition of a on-screen-shell
*/

#ifndef _SHELL_COMMAND_H
#define _SHELL_COMMAND_H

#include "base_object.h"

#include <stdarg.h>

typedef enum ShellParameterType
{
  ShellParameterNull,
  ShellParameterChar,
  ShellParameterString,
  ShellParameterInt,
  ShellParameterUInt,
  ShellParameterFloat,
  /* ... */
};

// FORWARD DECLARATION
template<class T> class tList;
template<class T> class tIterator;
#define MAX_SHELL_COMMAND_SIZE

class ShellCommandBase
{
  public:
    static bool execute (const char* executionString);

  protected:
    ShellCommandBase(const char* commandName, ClassID classID);

    static bool isRegistered(const char* commandName, ClassID classID);


  private:
    char*                            commandName;         //!< The name of the Command when executed
    long                             classID;             //!< The ID of the Class asociated to this Command


    static tList<ShellCommandBase>*  commandList;         //!< A list of availiable commands.
};

//! keeps information about a ShellCommand
template<class T> class ShellCommand : public ShellCommandBase
{
  public:
    static void registerCommand(const char* commandName, void* pointerToFunction, ClassID classID, ShellParameterType param1Type, void* Object = NULL);

    static void unregisterCommand(const char* commandNaame, ClassID classID);

  private:
    ShellCommand(const char* command, void* pointerToFunction, ClassID classID, ShellParameterType param1Type, void* Object = NULL);


  public:
    void*                functionPointer;     //!< The pointer to the function of the Class (or static pointer if ClassID == CL_NULL )
    T*                   Object;              //!< The pointer to the object to apply this function to (NULL if classID == CL_NULL )
};

template<class T>
    ShellCommand<T>::ShellCommand(const char* commandName, void* pointerToFunction, ClassID classID, ShellParameterType param1Type, void* Object)
  : ShellCommandBase (command, classID)
{

}

template<class T>
    void ShellCommand<T>::registerCommand(const char* commandName, void* pointerToFunction, ClassID classID, ShellParameterType param1Type, void* Object)
{
  if (isRegistered() == true)
    return;
  else
  {
    ShellCommand<T>* newCommand = new ShellCommand<T>(command);
    elem->functionPointer = pointerToFunction;
    elem->parameter1 = param1Type;
    elem->Object = Object;


  }
}

#endif /* _SHELL_COMMAND_H */
